local utils = require "spec.utils"
local OpCode = require "lustre.frame.opcode"

describe("OpCode", function()
  describe("encode/decode", function()
    it("data-continue", function()
      local code = 0
      local oc = OpCode.decode(code)
      assert.are(oc.type, "data")
      assert.are(oc.sub, "continue")
      assert.are(oc.value, nil)
      assert.are(oc:encode(), code)
    end)
    it("data-text", function()
      local code = 1
      local oc = OpCode.decode(code)
      assert.are(oc.type, "data")
      assert.are(oc.sub, "text")
      assert.are(oc.value, nil)
      assert.are(oc:encode(), code)
    end)
    it("data-binary", function()
      local code = 2
      local oc = OpCode.decode(code)
      assert.are(oc.type, "data")
      assert.are(oc.sub, "binary")
      assert.are(oc.value, nil)
      assert.are(oc:encode(), code)
    end)
    it("data-reserved", function()
      for code = 3, 7 do
        local oc = OpCode.decode(code)
        assert.are(oc.type, "data")
        assert.are(oc.sub, "reserved")
        assert.are(oc.value, code)
        assert.are(oc:encode(), code)
      end
    end)
    it("data-control", function()
      local code = 8
      local oc = OpCode.decode(code)
      assert.are(oc.type, "control")
      assert.are(oc.sub, "close")
      assert.are(oc.value, nil)
      assert.are(oc:encode(), code)
    end)
    it("data-control", function()
      local code = 9
      local oc = OpCode.decode(code)
      assert.are(oc.type, "control")
      assert.are(oc.sub, "ping")
      assert.are(oc.value, nil)
      assert.are(oc:encode(), code)
    end)
    it("data-control", function()
      local code = 10
      local oc = OpCode.decode(code)
      assert.are(oc.type, "control")
      assert.are(oc.sub, "pong")
      assert.are(oc.value, nil)
      assert.are(oc:encode(), code)
    end)
    it("data-control", function()
      for code = 11, 15 do
        local oc = OpCode.decode(code)
        assert.are(oc.type, "control")
        assert.are(oc.sub, "reserved")
        assert.are(oc.value, code)
        assert.are(oc:encode(), code)
      end
    end)
    it("out of range", function()
      local oc = OpCode.decode(16)
      assert.are(oc, nil)
    end)
    it("bad values", function()
      local oc = OpCode.from("junk")
      local b, err = oc:encode()
      assert(not b, "expected no bytes")
      assert.are(err, "Invalid opcode")
    end)
    describe("type constructors", function()
      it("ping", function()
        local oc = OpCode.ping()
        assert.are(oc.type, "control")
        assert.are(oc.sub, "ping")
        assert.are(oc:encode(), 9)
      end)
      it("pong", function()
        local oc = OpCode.pong()
        assert.are(oc.type, "control")
        assert.are(oc.sub, "pong")
        assert.are(oc:encode(), 10)
      end)
      it("close", function()
        local oc = OpCode.close()
        assert.are(oc.type, "control")
        assert.are(oc.sub, "close")
        assert.are(oc:encode(), 8)
      end)
      it("continue", function()
        local oc = OpCode.continue()
        assert.are(oc.type, "data")
        assert.are(oc.sub, "continue")
        assert.are(oc:encode(), 0)
      end)
      it("text", function()
        local oc = OpCode.text()
        assert.are(oc.type, "data")
        assert.are(oc.sub, "text")
        assert.are(oc:encode(), 1)
      end)
      it("binary", function()
        local oc = OpCode.binary()
        assert.are(oc.type, "data")
        assert.are(oc.sub, "binary")
        assert.are(oc:encode(), 2)
      end)
    end)
  end)
end)
